import os
from django.conf import settings
from django.core.management.base import BaseCommand
from mako.template import Template
from djangox.apps import import_app, tools


class Command(BaseCommand):
    args = 'app name'
    help = '''Setup static folder for specified app.
 - create static folder
 - ready to use bower
   - generate .bowerrc and bower.json
   - bower packages will be installed in static/lib
   - bootstrap, font-awesome, jquery will be included in bower.json by default.
'''

    def handle(self, *args, **options):
        app_name = args[0]
        os.makedirs(app_name + '/' + 'static/lib', exist_ok=True)

        template_path = filename=tools.__path__[0] + '/setupstatic/'
        with open(app_name + '/../.bowerrc', 'w') as f:
            f.write(Template(filename=template_path + '.bowerrc').render(app_name=app_name))

        with open(app_name + '/../bower.json', 'w') as f:
            f.write(Template(filename=template_path + 'bower.json').render(app_name=app_name))

        print('Now you can use bower command. Try `bower install`.')




from django.conf.urls import patterns, include, url
from djangox.route import discover_controllers


urlpatterns = patterns('',
    url('', include('social.apps.django_app.urls', namespace='social')),
    (r'', discover_controllers('unilogin.controllers')),
)


from datetime import date
from fabric.context_managers import cd, settings
from fabric.operations import run, sudo
from fabric.contrib.files import exists
from fabric.decorators import task, roles
from fabric.state import env
from fabtools import require
import fabtools
from fabtools.python import virtualenv
import deployconfig


env.user = 'ubuntu'
env.home = '/home/' + env.user
env.project_path = env.home + '/' + env.project_name
env.static_path = env.home + '/static'
env.deploy_key_path = env.home + '/.ssh/' + 'id_deploy'
env.virtualenv = env.home + '/venv'
env.backup_dir = '/mnt/data'


def checkout(url, directory):
    if exists(directory):
        with cd(directory):
            run("ssh-agent bash -c 'ssh-add %s; git pull'" % env.deploy_key_path)

    else:
        with cd(env.home):
            run("ssh-agent bash -c 'ssh-add %s; git clone %s'" % (env.deploy_key_path, url))


@task
@roles('web')
def setup_web():
    require.deb.packages(['git', 'npm', 'nodejs-legacy', 'python3-dev', 'libxml2-dev', 'libxslt1-dev', 'libpq-dev'])
    require.nginx.server()
    require.nginx.site(env.server_name, template_source='nginx-site',
                       port=80,
                       server_alias='',
                       static_path=env.static_path)
    # require.nginx.disabled('default')

    with settings(warn_only=True):
        if run('type bower').return_code:
            sudo('npm install -g bower')

    update_source()

    require.directory(env.home + '/logs')
    require.python.packages(['uwsgi', 'virtualenv', 'ipython', 'celery'], use_sudo=True)
    require.file('/etc/init/uwsgi.conf', source='uwsgi.conf', use_sudo=True)
    # require.file('/etc/init/celery.conf', source='celery.conf', use_sudo=True)
    require.directory('/etc/uwsgi', use_sudo=True)
    require.files.template_file('/etc/uwsgi/%s.ini' % env.project_name,
                                template_source='uwsgi.ini',
                                context=env, use_sudo=True)
    require.service.started('uwsgi')
    require.directory(env.home + '/bin')
    require.files.template_file(env.home + '/bin/djangorc',
                                template_source='bin/djangorc',
                                context=env)
    require.files.template_file(env.home + '/bin/loadenv',
                                template_source='bin/loadenv',
                                context=env)
    run('chmod +x ' + env.home + '/bin/loadenv')

    # require.service.started('celery')


@task
@roles('db')
def setup_db():
    require.deb.packages(['postgresql', 'libpq-dev'])
    if not fabtools.postgres.user_exists(env.db_user):
        fabtools.postgres.create_user(env.db_user, env.db_password, createdb=True)

    if not fabtools.postgres.database_exists(env.project_name):
        fabtools.postgres.create_database(env.project_name, env.project_name)

    # require.python.package('awscli', use_sudo=True)
    # require.files.directory(env.home + '/.aws')
    # require.files.template_file(env.home + '/.aws',
    #                             template_source='.aws/config', context=env)
    # require.files.template_file(env.home + '/.aws',
    #                             template_source='.aws/credentials', context=env)
    # run('chmod 0600 ' + env.home + '/.aws/*')

    require.directory(env.backup_dir, use_sudo=True)
    sudo('chown ubuntu.ubuntu %s' % env.backup_dir)
    # require.files.template_file(env.home + '/bin/backup-postgresql.sh',
    #                             template_source='bin/backup-postgresql.sh',
    #                             context=env)
    # run('chmod +x ' + env.home + '/bin/backup-postgresql.sh')
    # fabtools.cron.add_task('backup-postgresql',
    #                        '0 0 * * *',
    #                        'ubuntu',
    #                        '%s/bin/backup-postgresql.sh' % env.home)


@task
@roles('web')
def deploy():
    update_source()
    require.service.restart('uwsgi')
    # require.service.restart('celery')


@roles('web')
def update_source():
    require.file(env.deploy_key_path, source='id_deploy')
    run('chmod 0600 ' + env.deploy_key_path)
    checkout(env.project_repo, env.project_path)
    require.files.template_file(env.project_path + '/%s/production.py' % env.project_name,
                                template_source='production.py.template', context=env)

    require.python.virtualenv(env.virtualenv, venv_python='/usr/bin/python3')
    with virtualenv(env.virtualenv):
        require.python.requirements(env.project_path + '/requirements.txt')
        with cd(env.project_path):
            run('bower install')
            run('./manage.py collectstatic --settings=%s.production --noinput' % env.project_name)
            run('./manage.py migrate --settings=%s.production --noinput' % env.project_name)



import distutils
from distutils.dir_util import copy_tree
from django.conf import settings
import importlib
import os
import re
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = "Install app specified by module name, and copy it's files into this project"

    def add_arguments(self, parser):
        parser.add_argument('app_name')

    def handle(self, app_name, *args, **options):
        import_app(app_name)


def import_app(app_name, edit_settings=True):
    print(app_name)
    try:
        app_module = importlib.import_module(app_name)
    except:
        print('cannot import %s' % app_name)
        return

    module_name_only = app_module.__name__.split('.')[-1]
    if edit_settings:
        insert_app_to_settings(app_name)

    copy_tree(app_module.__path__[0], settings.BASE_DIR + '/' + module_name_only)
    print('copied %s into %s' % (app_module.__path__[0], settings.BASE_DIR + '/' + module_name_only))


def insert_app_to_settings(app_name):
    project_path = os.path.dirname(os.path.normpath(os.sys.modules[settings.SETTINGS_MODULE].__file__))
    settings_edit = CodeEditor(project_path + os.sep + 'settings.py')
    settings_edit.insert_line("    '%s'," % app_name.split('.')[-1], after='INSTALLED_APPS')
    settings_edit.commit()
    print("edited " + project_path + os.sep + 'settings.py')


class CodeEditor(object):

    def __init__(self, filename):
        self.filename = filename
        source = open(filename, 'r').read()
        self.lines = source.splitlines()
        self.cursor = 0

    def insert_tuple_element(self, name, value):
        for index, line in enumerate(self.lines):
            if line.strip().startswith(name):
                insert_index = index + 1
                break

        self.lines.insert(insert_index, '    ' + repr(value) + ',')

    def go_line(self, expr):
        for index, l in enumerate(self.lines[self.cursor:], self.cursor):
            if expr in l:
                self.cursor = index
                break

    def replace_all(self, expr, replacement):
        for index, l in enumerate(self.lines, self.cursor):
            if expr in l:
                self.lines[index] = l.replace(expr, replacement)

    def replace_line(self, expr, replacement):
        for index, l in enumerate(self.lines[self.cursor:], self.cursor):
            if expr in l:
                self.cursor = index
                break

        self.lines[self.cursor] = self.lines[self.cursor].replace(expr, replacement)

    def insert_line(self, line, after):
        if line in self.lines: return

        insert_index = 0
        for index, l in enumerate(self.lines[self.cursor:], self.cursor):
            if re.match(after, l):
                insert_index = index + 1
                break;
        self.lines.insert(insert_index, line)

    def append_line(self, line):
        self.lines.append(line)

    def to_source(self):
        return '\n'.join(self.lines)

    def commit(self):
        open(self.filename, 'w').write(self.to_source())


from django.conf.urls import include, url
import inspect
import pkgutil
import re
import sys
import types

def discover_controllers(package, method_first=False):
    '''
    Discover controller functions within give package.
    '''
    if isinstance(package, types.ModuleType):
        package = package.__name__
        
    urls = []
    __import__(package)

    if hasattr(sys.modules[package], 'index'):
        urls.append(url('^$', getattr(sys.modules[package], 'index')))
    
    for _, name, _ in pkgutil.iter_modules([sys.modules[package].__path__[0]]):
        __import__(package + '.' + name)
        
        controller = sys.modules[package + '.' + name]
        
        for member in dir(controller):
            func = getattr(controller, member)
            if not inspect.isfunction(func): continue
            args = inspect.getargspec(func).args
            
            # TODO filter request functions
            # if len(args) == 0 or args[0] != 'request': continue

            if method_first:
                urls.append(url('^' + name + '/' + member + '/(?P<resource_id>[^/\?\&]+)/?$', func))
            else:
                urls.append(url('^' + name + '/(?P<resource_id>[^/\?\&]+)/' + member + '/?$', func))

            urls.append(url('^' + name + '/' + member + '/?$', func))

        if 'show' in dir(controller):
            urls.append(url('^' + name + '/(?P<resource_id>[^/\?\&]+)/?$', getattr(controller, 'show')))
        if 'index' in dir(controller):
            urls.append(url('^' + name + '$', getattr(controller, 'index')))
                
    return include(urls)

class SubdomainMiddleware(object):
    def process_request(self, request):
        hostname = request.get_host().split(':')[0]
        
        name_parts = hostname.split('.')
        
        if len(name_parts) < 2:
            request.subdomain = None
        else:
            request.subdomain = name_parts[0]
            
        return None



from django.core.management.base import BaseCommand
from djangox.apps import import_app


class Command(BaseCommand):
    help = 'setup deploy environment'

    def handle(self, *args, **options):
        import_app('djangox.deploy', edit_settings=False)


from django.conf.urls import include, url
import inspect
import pkgutil
import re
import sys
import types
from django.utils.decorators import classonlymethod
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View


class RESTController(object):

    @classonlymethod
    def as_view(cls):

        @csrf_exempt
        def dispatch(request, resource_id=None, action=None):
            self = cls()
            method = request.method.lower()

            if action:
                return getattr(self, action)(request, resource_id)

            elif method == 'get':
                if resource_id:
                    return self.show(request, resource_id)
                else:
                    return self.index(request)
            elif method == 'post':
                return self.create(request)
            elif method == 'put':
                return self.update(request, resource_id)
            elif method == 'delete':
                return self.delete(request, resource_id)
            elif method == 'head':
                return self.info(request)
            else:
                raise Exception('No route found: %s %s' % (method, resource_id))

        return dispatch


def discover(package, method_first=False):
    '''
    Discover controller functions within give package.
    '''
    if isinstance(package, types.ModuleType):
        package = package.__name__

    urls = []
    __import__(package)
    for module_loader, name, ispkg in pkgutil.walk_packages(sys.modules[package].__path__):
        module = module_loader.find_module(name).load_module(name)

        for member in dir(module):
            controller_class = getattr(module, member)
            if inspect.isclass(controller_class) and issubclass(controller_class, RESTController):
                resource_name = controller_class.__name__.replace('Controller', '').lower()
                urls.append(url(resource_name + '/(?P<resource_id>[^/\?\&]+)?',
                                controller_class.as_view()))

    return include(urls)



import urllib.parse
from django.conf import settings
from django.contrib import auth
from django.contrib.auth import logout as auth_logout
from django.contrib.auth.models import User
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.template.context import RequestContext
from django.template.defaulttags import csrf_token, CsrfTokenNode
from djangox.mako import render_to_response


def login(request):
    request.session['login_error_url'] = request.build_absolute_uri()

    next = request.GET.get('next', '/')

    social_auth_providers = settings.SOCIAL_AUTH_PROIVDERS
    print('rc', RequestContext(request))
    return render(request, 'unilogin/login.html', locals())


def provider_authorize(request):
    request.session['state'] = request.GET['state']
    request.session['host'] = request.GET['host']

    next = reverse(provider_complete)

    if request.user.is_authenticated() and not request.GET['connect_more']:
        return HttpResponseRedirect(next)

    social_auth_providers = settings.SOCIAL_AUTH_PROIVDERS
    return render_to_response('accounts/login.html', locals(), RequestContext(request))


def provider_complete(request):
    if not request.user.is_authenticated():
        raise Exception('authentication failed')

    host = request.session['host']
    state = request.session['state']

    del request.session['host']
    del request.session['state']

    return HttpResponseRedirect('http://%s%s?%s' % (
        host,
        reverse(consumer_complete),
        urllib.parse.urlencode({
            'user': request.user.id,
            'state': state
        }))
    )


def consumer_complete(request):
    user = User.objects.get(id=request.GET['user'])
    next = request.session['next']
    state = request.session['state']

    request.session.clear()

    if state != request.GET['state']:
        raise Exception('state token mismatch: %s != %s' % (state, request.GET['state']))

    user.backend = 'django.contrib.auth.backends.ModelBackend'
    auth.login(request, user)
    return HttpResponseRedirect(next)


def logout(request):
    """Logs out user"""
    auth_logout(request)

    return HttpResponseRedirect(request.GET.get('next', '/'))
    


from django.core.management.base import BaseCommand
from djangox.apps import import_app


class Command(BaseCommand):
    args = 'package name of app'
    help = 'copy specified app files into this project.'

    def handle(self, *args, **options):
        import_app(args[0])


from setuptools import setup, find_packages

setup(name='djangox',
      description='Another way of using django. Convention over configuration for urls, mako template language support,'
                  'json response, REST style API, convenient scripts, ...',
      author='Youngrok Pak',
      author_email='pak.youngrok@gmail.com',
      keywords= 'rest route autodiscover django djangox mako',
      url='https://github.com/youngrok/djangox',
      version='0.1.1',
      packages=find_packages(),
      package_data={'djangox.apps.bs4templates': ['static/*', 'templates/*', 'templates/bs4templates/*'],
                    'djangox.apps.tools': ['setupstatic/*'],
                    'djangox.apps.model.templates': ['common/*', 'sample/*'],
                    'djangox.apps.deploy': ['*', 'bin/*'],
                    },

      classifiers = [
                     'Development Status :: 3 - Alpha',
                     'Topic :: Software Development :: Libraries',
                     'License :: OSI Approved :: BSD License']
      )



import os
import sys
import traceback
import types
import warnings
from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse, get_resolver, get_urlconf
from django.http import HttpResponse, HttpResponseServerError
from django.shortcuts import render
from django.template.backends.base import BaseEngine
from django.template.base import TemplateDoesNotExist
from django.template.context import RequestContext, Context, make_context
from django.template.engine import Engine, _dirs_undefined
from django.templatetags import static
from django.utils.deprecation import RemovedInDjango20Warning
from django.utils.translation import ugettext
from mako import exceptions
from mako.exceptions import TemplateLookupException
from mako.lookup import TemplateLookup
from mako.template import Template


def url(view_name, *args, **kwargs):
    try:
        return reverse(view_name, args=args, kwargs=kwargs)
    except:
        resolver = get_resolver(get_urlconf())
        for key in resolver.reverse_dict.keys():
            if isinstance(key, types.FunctionType):
                name = key.__module__ + '.' + key.__name__
            else:
                name = key
            
            if name.endswith(view_name):
                return reverse(name, args=args, kwargs=kwargs)
        
        raise
        

default_context = {
    'url' : url,
    'static': static.static,
    '_': ugettext,
}

default_charset = getattr(settings, 'DEFAULT_CHARSET', 'utf8')

app_template_dirs = []
fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()

for app in settings.INSTALLED_APPS:
    try:
        mod = import_module(app)
    except ImportError as e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))
    template_dir = os.path.join(os.path.dirname(mod.__file__), 'templates')
    if os.path.isdir(template_dir):
        app_template_dirs.append(template_dir)


template_lookup = TemplateLookup(directories=app_template_dirs, 
                                 input_encoding=default_charset, 
                                 output_encoding=default_charset, 
                                 )

def render_to_response(filename, dictionary, context_instance=None):
    '''
    :param filename:
    :param dictionary:
    :param context_instance:
    :return: rendered django HttpResponse
    '''

    dictionary.update(default_context)

    if context_instance:
        return render(context_instance.request, filename, dictionary)

    if hasattr(settings, 'MAKO_DEFAULT_CONTEXT'):
        dictionary.update(settings.MAKO_DEFAULT_CONTEXT)

    try:
        template = template_lookup.get_template(filename)
        return HttpResponse(template.render(**dictionary))
    except exceptions.TopLevelLookupException:
        raise
    except:
        traceback.print_exc()
        return HttpResponseServerError(exceptions.html_error_template().render())


class MakoTemplateEngine(BaseEngine):

    app_dirname = 'templates'

    def __init__(self, params):
        params = params.copy()
        options = params.pop('OPTIONS').copy()
        options.setdefault('debug', settings.DEBUG)
        options.setdefault('file_charset', settings.FILE_CHARSET)
        super().__init__(params)
        self.apps = options.get('apps', None)
        self.app_dirs = []
        for app in self.apps:
            app_module = import_module(app)
            self.app_dirs.append(os.path.abspath(app_module.__path__[0]))

    def from_string(self, template_code):
        return MakoTemplateWrapper(Template(text=template_code))

    def get_template(self, template_name, dirs=_dirs_undefined):
        try:
            mt = template_lookup.get_template(template_name)

            if not [dir for dir in self.app_dirs if os.path.join(dir, 'templates', template_name) == mt.filename]:
                raise TemplateDoesNotExist("template does not exists in templates directories in specified apps: %s", str(self.apps))
        except TemplateLookupException as e:
            raise TemplateDoesNotExist(str(e))

        return MakoTemplateWrapper(mt)


class MakoTemplateWrapper(object):

    def __init__(self, template):
        self.template = template
        self.engine = Engine.get_default()

    @property
    def origin(self):
        # TODO: define the Origin API. For now simply forwarding to the
        #       underlying Template preserves backwards-compatibility.
        return self.template

    def render(self, context=None, request=None):
        # A deprecation path is required here to cover the following usage:
        # >>> from django.template import Context
        # >>> from django.template.loader import get_template
        # >>> template = get_template('hello.html')
        # >>> template.render(Context({'name': 'world'}))
        # In Django 1.7 get_template() returned a django.template.Template.
        # In Django 1.8 it returns a django.template.backends.django.Template.
        # In Django 2.0 the isinstance checks should be removed. If passing a
        # Context or a RequestContext works by accident, it won't be an issue
        # per se, but it won't be officially supported either.
        if isinstance(context, RequestContext):
            if request is not None and request is not context.request:
                raise ValueError(
                    "render() was called with a RequestContext and a request "
                    "argument which refer to different requests. Make sure "
                    "that the context argument is a dict or at least that "
                    "the two arguments refer to the same request.")
            warnings.warn(
                "render() must be called with a dict, not a RequestContext.",
                RemovedInDjango20Warning, stacklevel=2)

        elif isinstance(context, Context):
            warnings.warn(
                "render() must be called with a dict, not a Context.",
                RemovedInDjango20Warning, stacklevel=2)

        else:
            context = make_context(context, request)

        if hasattr(settings, 'MAKO_DEFAULT_CONTEXT'):
            context.dicts.append(settings.MAKO_DEFAULT_CONTEXT)

        context.dicts.append(default_context)
        try:
            with context.bind_template(self):
                return self.template.render(**context.flatten())
        except exceptions.TopLevelLookupException:
            raise
        except:
            traceback.print_exc()
            return exceptions.html_error_template().render()




from fabric.state import env

env.roledefs.update({
    'web': [
        'webserver1.com',
        'webserver2.com'
    ],
    'db': ['dbserver1.com']
})

env.server_name = 'mydomain.com'
env.project_name = 'myproject'
env.project_repo = 'git@github.com:me/myproject.git'

env.key_filename = 'mykey.pem'
env.backup_bucket = 'myproject-backup'

env.db_host = 'dbserver1.com'
env.db_name = 'mydbname'
env.db_user = 'mydbuser'
env.db_password = 'mydbpassword'
